/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.farm.fieldplan.wizard;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.model.FieldProvider;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;

/**
 * Panel which displays fields to add to a field plan
 *
 * @author Merijn Zengers
 */
@NbBundle.Messages({"projects_label=Field-projects"})
public class FieldPlanFieldsWizardPanelVisual extends ASBodyPanel implements ListSelectionListener {

    private ImageIcon titleIcon = new ImageIcon(ImageUtilities.loadImage("nl/cloudfarming/client/farm/fieldplan/Plant32.png"));
    private JList fieldProjects;
    private DefaultListModel<FieldProvider> fieldProjectModel;
    private JScrollPane fieldScrollPane;
    private ShuttleBox shuttleBox;
    private JLabel projectLabel;
    private static final int LIST_HEIGHT = 300;
    private static final int LIST_WIDHT = 150;
    private static final int BORDER_THICKNESS = 1;
    private static final Color BORDER_COLOR = Color.black;
    public static final String PROP_SELECTED_FIELDS = "selectedFields";

    public FieldPlanFieldsWizardPanelVisual() {
        super(new MigLayout("wrap 2"));
        init();
    }

    private void init() {
        setIcon(titleIcon);
        projectLabel = new JLabel(NbBundle.getMessage(FieldPlanFieldsWizardPanelVisual.class, "projects_label"));

        Font old = projectLabel.getFont();
        projectLabel.setFont(new Font(old.getFontName(), Font.BOLD, old.getSize()));

        fieldProjectModel = new DefaultListModel<>();
        fieldProjects = new JList(fieldProjectModel);
        fieldProjects.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        fieldProjects.addListSelectionListener(this);

        fieldScrollPane = new JScrollPane(fieldProjects);
        fieldScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        fieldScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        fieldScrollPane.setBorder(new LineBorder(BORDER_COLOR, BORDER_THICKNESS));
        fieldScrollPane.setPreferredSize(new Dimension(LIST_WIDHT, LIST_HEIGHT));

        shuttleBox = new ShuttleBox();
        this.add(projectLabel);
        this.add(shuttleBox, "span 1 2");
        this.add(fieldScrollPane, "align left top");

    }

    /**
     * Gets the selected fields
     *
     * @return list of selected fields
     */
    public List<Field> getSelectedFields() {
        return shuttleBox.getSelectedFields();
    }

    /**
     * Adds a fieldProvider to the list of available providers
     *
     * @param provider the provider to add
     */
    public void addFieldProvider(FieldProvider provider) {
        if (!this.fieldProjectModel.contains(provider)) {
            this.fieldProjectModel.addElement(provider);
        }
    }

    /**
     * Set the available fields
     *
     * @param fields
     */
    public void setAvailableFields(List<Field> fields) {
        shuttleBox.setAvailableFields(fields);
    }

    @Override
    public void valueChanged(ListSelectionEvent lse) {
        shuttleBox.setAvailableFields(((FieldProvider) fieldProjects.getSelectedValue()).getFields());
    }

    /**
     * custom ShuttleBox component
     */
    @NbBundle.Messages({"leftList_label=Fields in project:",
        "rightList_label=Selected fields:"})
    private class ShuttleBox extends JPanel implements ActionListener {

        private FieldListModel availableListModel;
        private FieldListModel selectedListModel;
        private JLabel availableListLabel;
        private JLabel selectedListLabel;
        private JScrollPane availableScrollPane;
        private JScrollPane selectedScrollPane;
        private JList availableList;
        private JList selectedList;
        private JButton addButton;
        private JButton removeButton;
        private JPanel buttonPanel;
        private final String LEFT_LIST_TITLE = NbBundle.getMessage(ShuttleBox.class, "leftList_label");
        private final String RIGHT_LIST_TITLE = NbBundle.getMessage(ShuttleBox.class, "rightList_label");
        private static final int BUTTON_WIDTH = 25;
        private static final int BUTTON_HEIGHT = 25;
        private static final int BUTTON_INSETS = 0;
        private static final String ADD_ICON = "+";
        private static final String REMOVE_ICON = "-";

        public ShuttleBox() {
            init();
        }

        private void init() {
            this.setLayout(new MigLayout("wrap 3, insets 0 0 0 0"));
            this.setBackground(Color.white);

            availableListLabel = new JLabel(LEFT_LIST_TITLE);
            selectedListLabel = new JLabel(RIGHT_LIST_TITLE);

            Font old = availableListLabel.getFont();
            availableListLabel.setFont(new Font(old.getFontName(), Font.BOLD, old.getSize()));

            selectedListLabel.setFont(new Font(old.getFontName(), Font.BOLD, old.getSize()));

            availableListModel = new FieldListModel();
            availableList = new JList(availableListModel);

            availableScrollPane = new JScrollPane(availableList);
            availableScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
            availableScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
            availableScrollPane.setBorder(new LineBorder(BORDER_COLOR, BORDER_THICKNESS));
            availableScrollPane.setPreferredSize(new Dimension(LIST_WIDHT, LIST_HEIGHT));

            selectedListModel = new FieldListModel();
            selectedList = new JList(selectedListModel);
            selectedScrollPane = new JScrollPane(selectedList);
            selectedScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
            selectedScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
            selectedScrollPane.setBorder(new LineBorder(BORDER_COLOR, BORDER_THICKNESS));
            selectedScrollPane.setPreferredSize(new Dimension(LIST_WIDHT, LIST_HEIGHT));

            buttonPanel = new JPanel(new MigLayout("wrap 1"));
            buttonPanel.setBackground(Color.white);

            addButton = new JButton(ADD_ICON);
            removeButton = new JButton(REMOVE_ICON);

            addButton.setPreferredSize(new Dimension(BUTTON_WIDTH, BUTTON_HEIGHT));
            addButton.setMargin(new Insets(BUTTON_INSETS, BUTTON_INSETS, BUTTON_INSETS, BUTTON_INSETS));
            addButton.addActionListener(this);
            removeButton.setPreferredSize(new Dimension(BUTTON_WIDTH, BUTTON_HEIGHT));
            removeButton.setMargin(new Insets(BUTTON_INSETS, BUTTON_INSETS, BUTTON_INSETS, BUTTON_INSETS));
            removeButton.addActionListener(this);

            buttonPanel.add(addButton, "align center bottom");
            buttonPanel.add(removeButton, "align center top");


            this.add(availableListLabel, "span 2");
            this.add(selectedListLabel);
            this.add(availableScrollPane);
            this.add(buttonPanel, "center");
            this.add(selectedScrollPane);
        }

        /**
         * Gets all the fields from the available list
         *
         * @return list of fields.
         */
        public List<Field> getAvailableFields() {
            return availableListModel.getFields();
        }

        /**
         * Gets all the fields from the selected list
         *
         * @return list of Fields
         */
        public List<Field> getSelectedFields() {
            return selectedListModel.getFields();
        }

        /**
         * Set the available fields
         *
         * @param fields the fields to set
         */
        public void setAvailableFields(List<Field> fields) {
            availableListModel.clear();
            availableList.getSelectionModel().clearSelection();
            for (Field field : fields) {    
                availableListModel.add(field);
            }
        }

        /**
         * add a Field to the available list
         *
         * @param field the field to add
         */
        public void addAvailableField(Field field) {
            selectedListModel.add(field);
        }

        /**
         * Adds all the fields in the provided list
         *
         * @param fields
         */
        public void addSelectedFields(List<Field> fields) {
            selectedListModel.addAll(fields);
        }

        /**
         * Removes all the provided fields from the selected list
         *
         * @param fields the fields to remove
         */
        public void removeSelectedFields(List<Field> fields) {
            selectedListModel.removeElements(fields);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            if (ae.getSource() == addButton) {
                if (availableList.getSelectedValuesList() != null && !availableList.getSelectedValuesList().isEmpty()) {
                    addSelectedFields(availableList.getSelectedValuesList());
                }
            } else if (ae.getSource() == removeButton) {
                if (selectedList.getSelectedValuesList() != null && !selectedList.getSelectedValuesList().isEmpty()) {
                    removeSelectedFields(selectedList.getSelectedValuesList());
                }
            }
        }
    }

    private class FieldListModel extends AbstractListModel<Field> {

        List<Field> model;

        public FieldListModel() {
            model = new ArrayList<>();
        }

        @Override
        public int getSize() {
            return model.size();
        }

        @Override
        public Field getElementAt(int index) {
            return model.get(index);
        }

        public List<Field> getFields() {
            return Collections.unmodifiableList(model);
        }

        public void add(Field element) {
            if (!contains(element) && model.add(element)) {
                fireIntervalAdded(this, 0, getSize());
                fireContentsChanged(this, 0, getSize());
            }
        }

        public void addAll(List<Field> fields) {
            for (Field field : fields) {
                this.add(field);
            }
        }

        public void removeElements(List<Field> fields) {
            model.removeAll(fields);
            for (Field field : fields) {
                removeElement(field);
            }
        }

        public void clear() {
            model.clear();
            fireIntervalRemoved(this, 0, 0);
        }

        public boolean contains(Field element) {
            return model.contains(element);
        }

        public Iterator iterator() {
            return model.iterator();
        }

        public void removeElement(Field element) {
            model.remove(element);
            fireIntervalRemoved(this, 0, getSize());
            fireContentsChanged(this, 0, getSize());
        }
    }
}
